`timescale 1ns/1ns
//////////////////////////////////////////////////////////////////////////////
module	command_decoder		//
(input	wire	[7:0]	CODE	// Instruction
,input	wire		EN	// Enable decoding
//////////////////////////////////
,output	wire		QREG	// A0..A9 / B0..B9 / C0..C9 / D0..D9
,output	wire		QARG	// AA..AD / BA..BD / CA..CD / DA..DD
,output	wire		QEXT	//   AE   /   BE   /   CE   /   DE
,output	wire		QFIX	//   AF   /   BF   /   CF   /   DF
,output	wire		QRET	// E0..E7
,output	wire		QCND	// E8..EF
,output	wire		QHLT	// F0..FF /   00
,output	wire		QBCD	// 01::99
,output	wire		QREP	// 10::90
,output	wire		QALU	// 0A..9F
,output	wire		SHFT	// 0C..9F - Bits shifting
);
	wire		HREG, LREG;
	wire		HNUM, LNUM;

	function	is_reg;
	input	B3, B2, B1;
		begin
			is_reg = B3 & (B2 ^ B1);
		end
	endfunction

	function	is_num;
	input	B3, B2, B1;
		begin
			is_num = ~(B3 & (B2 | B1));
		end
	endfunction

	assign	HREG = is_reg(CODE[7], CODE[6], CODE[5]);
	assign	LREG = is_reg(CODE[3], CODE[2], CODE[1]);
	assign	HNUM = is_num(CODE[7], CODE[6], CODE[5]);
	assign	LNUM = is_num(CODE[3], CODE[2], CODE[1]);

	assign	QREG = EN & (HREG & LNUM);
	assign	QARG = EN & (HREG & LREG);
	assign	QEXT = EN & (HREG & CODE[3] & CODE[2] & CODE[1] & ~CODE[0]);
	assign	QFIX = EN & (HREG & CODE[3] & CODE[2] & CODE[1] & CODE[0]);
	assign	QRET = EN & (CODE[7] & CODE[6] & CODE[5] & ~CODE[4] & ~CODE[3]);
	assign	QCND = EN & (CODE[7] & CODE[6] & CODE[5] & ~CODE[4] & CODE[3]);
	assign	QHLT = EN & ((CODE[7] & CODE[6] & CODE[5] & CODE[4]) | ~(CODE[7] | CODE[6] | CODE[5] | CODE[4] | CODE[3] | CODE[2] | CODE[1] | CODE[0]));
	assign	QBCD = EN & (HNUM & LNUM & (CODE[3] | CODE[2] | CODE[1] | CODE[0]));
	assign	QREP = EN & (HNUM & (CODE[7] | CODE[6] | CODE[5] | CODE[4]) & ~(CODE[3] | CODE[2] | CODE[1] | CODE[0]));
	assign	QALU = EN & (HNUM & CODE[3] & (CODE[2] | CODE[1]));
	assign	SHFT = EN & (HNUM & CODE[3] & CODE[2]);
endmodule
//////////////////////////////////////////////////////////////////////////////
module	regfile
(input	wire		clk	// 
,input	wire		reset	// 
,input	wire		fetch	// Fetch-cycle
,input	wire		we0	// Write Enable to register#0
,input	wire		we1	// Write Enable to operand#1
//////////////////////////////////
,input	wire		is_reg	// Select the register
,input	wire		is_arg	// Select the operands
//////////////////////////////////
,input	wire	[7:0]	code	// Instruction Code
//////////////////////////////////
,input	wire	[7:0]	wd0	// Writing Data to register#0
,input	wire	[7:0]	wd1	// Writing Data to operand#1
//////////////////////////////////
,output	wire	[7:0]	rd0	// Reading Data from register#0
,output	wire	[7:0]	rd1	// Reading Data from operand#1
,output	wire	[7:0]	rd2	// Reading Data from operand#2
);
//////////////////////////////////
	reg	[7:0]	apr[0:31];
endmodule
//////////////////////////////////////////////////////////////////////////////
module	register_bar		//
(input	wire		CLOCK	//
,input	wire		RESET	//
,input	wire		WRITE0	// Writting to REG0 enable
,input	wire		WRITE	// Writting to REGi enable
//////////////////////////////////
,input	wire		WRITING	//
//////////////////////////////////
,input	wire	[7:0]	OPTION	// Optional data input
,input	wire	[7:0]	RESULT	// Writting data
,input	wire	[3:0]	RINDEX	// Receiver Index
,input	wire	[3:0]	TINDEX	// Translator Index
//////////////////////////////////
,output	wire	[7:0]	DATA0	// Optional data output
,output	wire	[7:0]	DATAR	// Receiver data
,output	wire	[7:0]	DATAT	// Translator data
//////////////////////////////////
);
parameter		DELAY = 0;
parameter		BARID = 8'h41;
//////////////////////////////////
	integer		i, a, b;
//////////////////////////////////
	reg	[7:0]	R0, R1, R2, R3, R4, R5, R6, R7, R8, R9;
	wire	[7:0]	R[0:9];
	assign		{R[0], R[1], R[2], R[3], R[4], R[5], R[6], R[7], R[8], R[9]} = {R0, R1, R2, R3, R4, R5, R6, R7, R8, R9};
	assign		DATA0 = R0;
	assign		DATAR = R[RINDEX];
	assign		DATAT = R[TINDEX];
	always @(posedge CLOCK)
	begin
		if(RESET)
		begin
			R0 <= 0;
			$display("REG0 Reset");
		end else begin
			if(WRITE)
			begin
				case(RINDEX)
				9:	R9 <= RESULT;
				8:	R8 <= RESULT;
				7:	R7 <= RESULT;
				6:	R6 <= RESULT;
				5:	R5 <= RESULT;
				4:	R4 <= RESULT;
				3:	R3 <= RESULT;
				2:	R2 <= RESULT;
				1:	R1 <= RESULT;
				0:	R0 <= RESULT;
				endcase
			end
			if(WRITE0 && (RINDEX != 0 || ~WRITE))
			begin
				R0 <= OPTION;
			end
		end
		if(WRITING)
			#DELAY	Logging;
	end
	task Logging;
	begin
		$write("%c", BARID);
		for(i = 0; i <= 9; i = i + 1)
		begin
			a = (TINDEX == i ? 8'h3D : 32);
			b = (RINDEX == i ? 8'h3D : 32);
			$write("%c%02X%c", a, R[i], b);
		end
		$display;
	end
	endtask
endmodule
//////////////////////////////////////////////////////////////////////////////
module	register_file
(input	wire		CLOCK	// 
,input	wire		RESET	//
,input	wire		ALUST	// ALU-status store to A0
,input	wire		PTRST	// Pointer store to B0:C0
,input	wire		STORE	// Result store
//////////////////////////////////
,input	wire	[7:0]	RESULT	// Writting result
,input	wire	[7:0]	STATUS	// ALU-Flags input
,input	wire	[15:0]	PTR	// Instruction Pointer
,input	wire	[3:0]	DSTIA	// Register index of A
,input	wire	[3:0]	DSTIB	// Register index of B
,input	wire	[3:0]	DSTIC	// Register index of C
,input	wire	[3:0]	DSTID	// Register index of D
,input	wire	[3:0]	SRCIA	// Register index of A
,input	wire	[3:0]	SRCIB	// Register index of B
,input	wire	[3:0]	SRCIC	// Register index of C
,input	wire	[3:0]	SRCID	// Register index of D
,input	wire	[3:0]	PTRI	// Pointer Bi:Ci-index
,input	wire	[1:0]	DST_RG	// Destinated Register Group
,input	wire	[1:0]	SRC_RG	// Source Register Group
//////////////////////////////////
,output	wire	[7:0]	FLAGS	// ALU-Flags output
,output	wire	[15:0]	PTRN	// Pointer Bi:Ci-address
,output	wire	[7:0]	DST
,output	wire	[7:0]	SRC
);
	wire	[7:0]	R[0:3];	// Register-Bars Ci, Di, Ai, Bi
	wire	[7:0]	T[0:3];	// Register-Bars Ci, Di, Ai, Bi
	assign		DST = R[DST_RG];
	assign		SRC = T[SRC_RG];
	register_bar BarA
	(.CLOCK(CLOCK)	,.RESET(RESET)
	,.RESULT(RESULT),.WRITING(STORE | ALUST | PTRST)
	,.WRITE0(ALUST)	,.OPTION(STATUS)
	,.RINDEX(DSTIA)	,.TINDEX(SRCIA)
	,.DATA0(FLAGS)
	,.DATAR(R[2])	,.DATAT(T[2])
	,.WRITE(DST_RG == 2 ? STORE : 1'b0)
	);
	register_bar BarB
	(.CLOCK(CLOCK)	,.RESET(RESET)
	,.RESULT(RESULT),.WRITING(STORE | ALUST | PTRST)
	,.WRITE0(PTRST)	,.OPTION(PTR[15:8])
	,.RINDEX(DSTIB)	,.TINDEX(SRCIB)
	,.DATA0(PTRN[15:8])
	,.DATAR(R[3])	,.DATAT(T[3])
	,.WRITE(DST_RG == 3 ? STORE : 1'b0)
	);
	register_bar BarC
	(.CLOCK(CLOCK)	,.RESET(RESET)
	,.RESULT(RESULT),.WRITING(STORE | ALUST | PTRST)
	,.WRITE0(PTRST)	,.OPTION(PTR[7:0])
	,.RINDEX(DSTIC)	,.TINDEX(SRCIC)
	,.DATA0(PTRN[7:0])
	,.DATAR(R[0])	,.DATAT(T[0])
	,.WRITE(DST_RG == 0 ? STORE : 1'b0)
	);
	register_bar BarD
	(.CLOCK(CLOCK)	,.RESET(RESET)
	,.RESULT(RESULT),.WRITING(STORE | ALUST | PTRST)
	,.WRITE0(1'b0)	,.OPTION(8'h00)
	,.RINDEX(DSTID)	,.TINDEX(SRCID)
	,.DATAR(R[1])	,.DATAT(T[1])
	,.WRITE(DST_RG == 1 ? STORE : 1'b0)
	);
	defparam BarA.DELAY = 1;
	defparam BarA.BARID = 8'h41;
	defparam BarB.DELAY = 2;
	defparam BarB.BARID = 8'h42;
	defparam BarC.DELAY = 3;
	defparam BarC.BARID = 8'h43;
	defparam BarD.DELAY = 4;
	defparam BarD.BARID = 8'h44;
	always	@(posedge CLOCK)
		if(ALUST | PTRST | STORE)
			#0 $display("_ _0  _1  _2  _3  _4  _5  _6  _7  _8  _9");
endmodule
//////////////////////////////////////////////////////////////////////////////
module	conditions
(input	wire	[7:0]	FLAGS	// ??-??-??-??-SF-PF-CF-ZF
,input	wire	[2:0]	CND	// PF, !PF, SF, !SF, CF, !CF, ZF, !ZF
,output	wire		EVENT	//
);
	wire		BITS[0:7];
	assign		BITS[0] = FLAGS[2];
	assign		BITS[1] = !FLAGS[2];
	assign		BITS[2] = FLAGS[3];
	assign		BITS[3] = !FLAGS[3];
	assign		BITS[4] = FLAGS[1];
	assign		BITS[5] = !FLAGS[1];
	assign		BITS[6] = FLAGS[0];
	assign		BITS[7] = !FLAGS[0];
	assign		EVENT = ~BITS[CND];
endmodule
//////////////////////////////////////////////////////////////////////////////
module	alu
(input	wire	[7:0]	DATA1	//
,input	wire	[7:0]	DATA2	//
,input	wire	[2:0]	MODE	// 2:ADC, 3:SBB, 4:AND, 5:OR, 6:XOR, 7:MOV
,input	wire	[3:0]	SHIFT	//
,input	wire		CARRIN	//
,output	wire	[7:0]	RESULT	//
,output	wire	[3:0]	FLAGS	//
);
	wire	[7:0]	RESULTS[0:7];
	wire	[7:0]	SHIFTS[0:9];
	wire		C2, C3;
	//
	assign		SHIFTS[0][7:0]	= {DATA2[7], DATA2[6], DATA2[5], DATA2[4], DATA2[3], DATA2[2], DATA2[1], DATA2[0]};
	assign		SHIFTS[1][7:0]	= {DATA2[6], DATA2[5], DATA2[4], DATA2[3], DATA2[2], DATA2[1], DATA2[0], DATA2[7]};
	assign		SHIFTS[2][7:0]	= {DATA2[5], DATA2[4], DATA2[3], DATA2[2], DATA2[1], DATA2[0], DATA2[7], DATA2[6]};
	assign		SHIFTS[3][7:0]	= {DATA2[4], DATA2[3], DATA2[2], DATA2[1], DATA2[0], DATA2[7], DATA2[6], DATA2[5]};
	assign		SHIFTS[4][7:0]	= {DATA2[3], DATA2[2], DATA2[1], DATA2[0], DATA2[7], DATA2[6], DATA2[5], DATA2[4]};
	assign		SHIFTS[5][7:0]	= {DATA2[2], DATA2[1], DATA2[0], DATA2[7], DATA2[6], DATA2[5], DATA2[4], DATA2[3]};
	assign		SHIFTS[6][7:0]	= {DATA2[1], DATA2[0], DATA2[7], DATA2[6], DATA2[5], DATA2[4], DATA2[3], DATA2[2]};
	assign		SHIFTS[7][7:0]	= {DATA2[0], DATA2[7], DATA2[6], DATA2[5], DATA2[4], DATA2[3], DATA2[2], DATA2[1]};
	assign		SHIFTS[8]	= ~DATA2;
	assign		SHIFTS[9][7:0]	= {DATA2[0], DATA2[1], DATA2[2], DATA2[3], DATA2[4], DATA2[5], DATA2[6], DATA2[7]};
	assign		{C2, RESULTS[2]} = DATA1 + DATA2 + CARRIN;
	assign		{C3, RESULTS[3]} = DATA1 - DATA2 - CARRIN;
	assign		RESULTS[4] = DATA1 & SHIFTS[SHIFT];
	assign		RESULTS[5] = DATA1 | SHIFTS[SHIFT];
	assign		RESULTS[6] = DATA1 ^ SHIFTS[SHIFT];
	assign		RESULTS[7] = SHIFTS[SHIFT];
	assign		RESULT = RESULTS[MODE];
	assign		FLAGS[3:0] = {RESULT[7], 1'b0, (MODE[2] ? 1'b0 : (MODE[1] ? C3 : C2)), (RESULT == 8'h00 ? 1'b1 : 1'b0)};
endmodule
//////////////////////////////////////////////////////////////////////////////
module	cpu
(input	wire		CLOCK	//
,input	wire		RESET	//
//////////////////////////////////
,inout	wire	[7:0]	DATA	//
//////////////////////////////////
,output	wire	[15:0]	ADDR	//
,output	reg		WRITE	//
,output	reg		READ	//
);
	reg		En;
	wire		Enable;
	wire		Qreg, Qarg, Qext, Qfix, Qret, Qcnd, Qhlt, Qbcd, Qrep, Qalu, Shft;
	wire	[7:0]	Result, Flags, AluFlags;
	reg	[3:0]	Cycle;	// Machine Cycle
	reg	[7:0]	Code;
	wire	[15:0]	PTRN;
	reg	[15:0]	IP;	// Instructions Pointer
	reg	[3:0]	IR;	// Instruction Repeats
	reg	[3:0]	Step;	// Instruction Skip
	//
	reg	[3:0]	RI[0:3];// Registers Index
	reg	[1:0]	RB;	// Receivers Bar
	reg	[1:0]	TB;	// Translators Bar
	wire	[7:0]	Dst, Src;
	wire	Break;
	//
	assign		ADDR = Cycle[2] | Cycle[3] ? PTRN : IP;
	assign		DATA = Cycle[2] & Qfix ? Dst : 'bZ;
	assign		AluFlags[7:4] = 4'b0000;
	assign		Break = ((Qreg || Qarg) && IR != 0) | Qret | Qhlt | Qbcd;
	//
	conditions Condition(.FLAGS(Flags), .CND(Code[2:0]), .EVENT(Enable));
	alu Alu(.DATA1(Dst), .DATA2(Src), .MODE(Code[2:0]), .SHIFT(IR), .CARRIN(Flags[1]), .RESULT(Result), .FLAGS(AluFlags[3:0]));
	//
	register_file Context
	(.CLOCK(CLOCK)	,.RESET(RESET)
	,.ALUST(Qalu & Cycle[1])
	,.PTRST(Break & Cycle[1])
	,.STORE((Qalu & Cycle[1]) | (Qext & Cycle[3]))
	,.RESULT(Qalu ? Result : DATA)
	,.STATUS(AluFlags)
	,.PTR(IP)
	,.DSTIA(RI[2])
	,.DSTIB(RI[3])
	,.DSTIC(RI[0])
	,.DSTID(RI[1])
	,.SRCIA(Qalu ? Code[7:4] : RI[2])
	,.SRCIB(Qalu ? Code[7:4] : RI[3])
	,.SRCIC(Qalu ? Code[7:4] : RI[0])
	,.SRCID(Qalu ? Code[7:4] : RI[1])
	,.PTRI(Qret ? Code[3:0] : IR)
	,.DST_RG(RB), .SRC_RG(TB)
	,.FLAGS(Flags)
	,.PTRN(PTRN)
	,.DST(Dst), .SRC(Src)
	);
	//
	command_decoder Decoder
	(.EN(~Cycle[0])
	,.CODE(Code)	,.QREG(Qreg)	,.QARG(Qarg)	,.QEXT(Qext)
	,.QFIX(Qfix)	,.QRET(Qret)	,.QCND(Qcnd)	,.QHLT(Qhlt)
	,.QBCD(Qbcd)	,.QREP(Qrep)	,.QALU(Qalu)	,.SHFT(Shft)
	);
	//
	always @(negedge CLOCK)
	begin
		if(RESET)
		begin
			Cycle <= 4'b0001;
			Code <= 8'h00;
		end else
		if(~Qalu || Shft || IR == 0 || Cycle[0])
		begin
			if((Cycle[1] & ~(Qext | Qfix)) | Cycle[3])
				Cycle <= 4'b0001;
			else
				Cycle <= Cycle << 1;
			if(Cycle[0] & READ)
				Code <= DATA;
		end
	end
	//
	always @(posedge CLOCK)
	begin
		if(RESET)
		begin
			WRITE <= 1'b0;
			READ <= 1'b0;
			IP <= 16'h0000;
			IR <= 4'b0000;
			$display("CPU:RESET");
		end else begin
			WRITE <= Cycle[2] & Qfix ? 1'b1 : 1'b0;
			READ <= (Cycle[2] & Qext) | Cycle[0] ? 1'b1 : 1'b0;
			if(Cycle[1])
			begin
				show();
				Step = 1;
				if(Qreg && IR == 0)
				// Rn (R0..R9)
					RI[Code[5:4]] <= Code[3:0];
				if(Qarg && IR == 0)
				// RT
					{RB, TB} <= {Code[5:4], Code[1:0]};
				if((Qreg || Qarg) && IR != 0)
				// n0 Rn | n0 RT
				begin
					Step = 0;
					IP <= {Code[7:0], IR[3:0], 4'h0};
				end
				if(Qret)
				// E0..E7
					IP <= PTRN + IR;
				if(Qcnd)
				// E8..EF
					if(Enable)
						Step = IR == 0 ? 2 : IR;
				if(Qhlt)
				// 00 F0..FF
					IP[15:0] <= {Code[7:0], IR[3:0], 4'h0};
				if(Qbcd)
				// nn (01..99)
					IP[15:0] <= 16'h0000;
				if(Qret | Qhlt | Qbcd)
				// 00 E0..E7 F0..FF 01..99
				begin
					IR <= 4'h0;
					Step = 0;
				end
				if(Qrep)
				// n0 (10..90)
					IR <= Code[7:4];
				else
				if(IR != 0 && Step != 0)
				begin
					$display("LAP %d", IR);
					IR <= IR - 1;
					Step = 0;
				end
				if(Step != 0)
					IP <= IP + Step;
			end else
			if(Cycle[3] & (Qext | Qfix))
				$display("[%04X]", ADDR);
		end
	end
	//////////////////////////////////////////////////////////////////////
	task	show;
	begin
		if(Qrep)
			$write("|%04X %02X  ", IP, Code);
		else
			$write("|%04X %c%02X ", IP, (IR == 0 ? 8'H20 : 8'H30 + IR), Code);
		if(Qreg)
			if(IR == 0)
				$display("REG %2X      ; Select %2X", Code, Code);
			else
				$display("HLT %02X%X0", Code, IR);
		if(Qarg)
			if(IR == 0)
				$display("ARG %X%X,%X%X   ; Use(%X%X,%X%X)", Code[7:4], RI[Code[5:4]], Code[3:0], RI[Code[1:0]], Code[7:4], RI[Code[5:4]], Code[3:0], RI[Code[1:0]]);
			else
				$display("HLT %02X%X0", Code, IR);
		if(Qext)	$display("EXT B%X:C%X   ; Extract [%04X] to %X%X", IR, IR, PTRN, Code[7:4], RI[Code[5:4]]);
		if(Qfix)	$display("FIX B%X:C%X   ; Fixing %X%X to [%04X]", IR, IR, Code[7:4], RI[Code[5:4]], PTRN);
		if(Qret)	$display("RET B%X:C%X%c%c ; JMP %04X", Code[3:0], Code[3:0], IR == 0 ? 8'h20 : 8'h2B, IR == 0 ? 8'h20 : 8'h30 + IR, PTRN + IR);
		if(Qcnd)
		begin
			$write("EI%X %c%c      ; Execute If ", Code[3:0], IR == 0 ? 8'h20 : 8'h23, IR == 0 ? 8'h20 : 8'h30 + IR);
			case(Code[2:0])
			0:	$display("po    (PF)");
			1:	$display("pe    (NP)");
			2:	$display("Minus (SF)");
			3:	$display("Plus  (NS)");
			4:	$display("Carry (CF)");
			5:	$display("Drain (NC)");
			6:	$display("Equal (ZF)");
			7:	$display("False (NZ)");
			endcase
		end
		if(Qhlt)	$display("HLT %02X%X0", Code, IR);
		if(Qbcd)	$display("BCD \"\\x%02X...\"", Code);
		if(Qrep)	$display("REP %X(...)", Code[7:4]);
		if(Qalu)
		begin
			case(Code[2:0])
			3'b010:	$write("ADD ");
			3'b011:	$write("SUB ");
			3'b100:	$write("AND ");
			3'b101:	$write("OR  ");
			3'b110:	$write("EOR ");
			3'b111:	$write("MOV ");
			endcase
			case(RB)
			0:	$write("C%X,", RI[RB]);
			1:	$write("D%X,", RI[RB]);
			2:	$write("A%X,", RI[RB]);
			3:	$write("B%X,", RI[RB]);
			endcase
			if(Shft && IR == 8)
				$write("NOT ");
			if(Shft && IR == 9)
				$write("REV ");
			case(TB)
			0:	$write("C%X", Code[7:4]);
			1:	$write("D%X", Code[7:4]);
			2:	$write("A%X", Code[7:4]);
			3:	$write("B%X", Code[7:4]);
			endcase
			if(Shft)
				case(IR)
				0:	$display;
				1:	$display("<<1");
				2:	$display("<<2");
				3:	$display("<<3");
				4:	$display("<<4");
				5:	$display(">>3");
				6:	$display(">>2");
				7:	$display(">>1");
				8:	$display;
				9:	$display;
				endcase
			else
			if(IR)
				$display("*%X", IR);
		end
	end
	endtask
endmodule
//////////////////////////////////////////////////////////////////////////////
module main;
	reg		Tick, Reset;
	reg	[16:0]	tmp;
	reg	[7:0]	RAM[0:65535];
	wire	[15:0]	ADDR;
	wire	[7:0]	DATA;
	wire		READ;
	wire		WRITE;
	//
	assign		DATA = READ ? RAM[ADDR] : 8'bZZZZZZZZ;
	//
	cpu Cpu(.CLOCK(Tick), .RESET(Reset), .DATA(DATA), .ADDR(ADDR), .READ(READ), .WRITE(WRITE));
	//
	always @(posedge WRITE)
	begin
		if(~Reset)
			RAM[ADDR] <= DATA;
		$write("%04X:", ADDR);
		for(tmp = 0; tmp < 16; tmp = tmp + 1)
			$write(" %02X", ADDR[3:0] == tmp ? RAM[(ADDR & 16'hFFF0) + tmp] : 8'hXX);
		$display(" %04X=%02X", ADDR, DATA);
	end
	always @(posedge READ)
	begin
		#5
		$write("%04X:", ADDR);
		for(tmp = 0; tmp < 16; tmp = tmp + 1)
			$write("%c%02X", (ADDR[3:0] == tmp ? 8'h5B : ADDR[3:0] == tmp - 1 ? 8'h5D : 8'h20), RAM[(ADDR & 16'hFFF0) + tmp]);
		$write("%c", ADDR[3:0] == tmp - 1 ? 8'h5D : 8'h20);
	end
	//
	always #25 Tick = ~Tick;
	//
	initial
	begin
		for(tmp = 0; tmp < 65536; tmp = tmp + 1)
			RAM[tmp] = 0;
		tmp = 64;
		$readmemh("cpu.vh", RAM, 0, 65535);
		Tick <= 0;
		Reset <= 1;
		#65 Reset <= 0;
		#6000 $finish;
	end
endmodule